# ================================================================
# Examples of the documentation of Model HT.n:
# HT.n-1.2: Exclusion vs. inclusion
# Data from Yonelinas (1994), Exp.2
#
# Date of creation: October, 2009
# Author: Siegfried Macho
# ================================================================
library(numDeriv)

# DIRECTORIES (PLEASE ADJUST TO YOUR PERSONAL SETTING)
DirSource <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/"			
DirSourceM <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/Models/"

# FILE NAME WITH THE MODEL 
Model <- "SDT-HT-n.R"

# FILE NAMES FOR ESTIMATION 
Main      <- "SDT-Main.R"
Auxiliary <- "SDT-Auxiliary.R"

# LOAD SOURCE FILES WITH SDT MODEL
source(paste(DirSourceM, Model, sep = ""))

# LOAD SOURCE FILES WITH AUXILIARY AND ESTIMATION FUNCTIONS
source(paste(DirSource, Auxiliary, sep = ""))
source(paste(DirSource, Main, sep = ""))

# DATA OF YONELINAS (1994), Experiment 2: SHORT LISTS
data.short <-   c(509, 495,  98,  30,  16,   4,   # Distractors
                  557, 263, 137,  78,  75,  42,   # Exclusion items
                   51, 109,  88, 105, 214, 585)   # Inclusion items

# DATA OF YONELINAS (1994), Experiment 2: LONG LISTS
data.long  <-   c(307, 530, 192,  79,  39,   5,  # Distractors
                  335, 308, 162, 178, 126,  43,  # Exclusion items
                   46, 179, 130, 189, 252, 356)  # Inclusion items
                   
n <- 3

#===============================================================================
#
# 1. HT.3 MODEL: 
#
#===============================================================================
# Specification of constraints
fixed.dist <- matrix(c(rep(0, 6),  1,                             # Distractors
                             1:6,  7), nr = 2, byrow = T)
                         
fixed.exc.inc <- matrix(c(rep(0, 6),        1,  1,                # Fixed constraints exclusion / inclusion
                              14:16, 20:22, 9, 18), nr = 2, byrow = T)
                        
fixed <- cbind(fixed.dist, fixed.exc.inc)
ident <- matrix(c(11:13,   8, 10,
                  25:23,  17, 19), nr = 2, byrow = T)

# Compute start parameter vector
par <- SDT.HT.n.start.par(n, data.short)

# Add redundant parameters so that itheir positions can be seen in the output
fixed.show <- HT.n.Redundant.to.Fixed(par, n, fixed, ident)

# Compute Parameter information
Par.Info <- SDT.Parameter.Info(par = par, n = n, Model.Id = "HT.n", fixed = fixed.show, ident = ident)
cat("\n-------------------------------------------------------------\n")
cat("HT.3: Yonelinas (1994), Exp.2, Parameter-Information:")
cat("\n-------------------------------------------------------------\n")
print(Par.Info)

cat("\n------------------------------------------------------------------------------------\n")
cat(" HT.3: Yonelinas (1994), Exp.2, short lists")
cat("\n------------------------------------------------------------------------------------\n")
Opti1.Obj <- SDT.Estimate(data = data.short, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = T)
Stat1.Obj <- SDT.Statistics(Opti1.Obj)
print(Stat1.Obj)

cat("\n------------------------------------------------------------------------------------\n")
cat(" HT.3: Yonelinas (1994), Exp.2, long lists")
cat("\n------------------------------------------------------------------------------------\n")
Opti1.Obj <- SDT.Estimate(data = data.long, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = T)
Stat1.Obj <- SDT.Statistics(Opti1.Obj)
print(Stat1.Obj)

#===============================================================================
#
# 2. HT.3 MODEL: FREE VARIANCE
#
#===============================================================================
fixed.exc.inc <- matrix(c(rep(0, 6),                       # Fixed constraints exclusion / inclusion
                              14:16, 20:22), nr = 2, byrow = T)
                        
fixed <- cbind(fixed.dist, fixed.exc.inc)
ident <- matrix(c(11:13,    8:9, 10,
                  25:23,  17:18, 19), nr = 2, byrow = T)

cat("\n------------------------------------------------------------------------------------\n")
cat(" HT.3: Yonelinas (1994), Exp.2, short lists, FREE VARIANCE PARAMETER")
cat("\n------------------------------------------------------------------------------------\n")
Opti1.Obj <- SDT.Estimate(data = data.short, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = T)
Stat1.Obj <- SDT.Statistics(Opti1.Obj)
print(Stat1.Obj)

cat("\n------------------------------------------------------------------------------------\n")
cat(" HT.3: Yonelinas (1994), Exp.2, long lists, FREE VARIANCE PARAMETER")
cat("\n------------------------------------------------------------------------------------\n")
Opti1.Obj <- SDT.Estimate(data = data.long, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = T)
Stat1.Obj <- SDT.Statistics(Opti1.Obj)
print(Stat1.Obj)

#===============================================================================
#
# 3. HT.2 MODEL: 
#
#===============================================================================

# Fixed constraints:
fixed.exc.inc <- matrix(c(rep(0, 8),        1,  1,                # Fixed constraints exclusion / inclusion
                              13:16, 20:23, 9, 18), nr = 2, byrow = T)
fixed <- cbind(fixed.dist, fixed.exc.inc)

# Equality constraints:
ident <- matrix(c(11:12,   8, 10,
                  25:24,  17, 19), nr = 2, byrow = T)

# Add redundant parameters so that their positions can be seen in the output
fixed.show <- HT.n.Redundant.to.Fixed(par, n, fixed, ident)

# Compute Parameter information
Par.Info.HT.2 <- SDT.Parameter.Info(par = par, n = n, Model.Id = "HT.n", fixed = fixed.show, ident = ident)
cat("\n-------------------------------------------------------------\n")
cat("HT.2: Kelley & Wixted, Exp.2, Parameter-Information:")
cat("\n-------------------------------------------------------------\n")
print(Par.Info.HT.2)

cat("\n------------------------------------------------------------------------------------\n")
cat(" HT.2: Yonelinas (1994), Exp.2, short lists")
cat("\n------------------------------------------------------------------------------------\n")
Opti1.Obj <- SDT.Estimate(data = data.short, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = T)
Stat1.Obj <- SDT.Statistics(Opti1.Obj)
print(Stat1.Obj)

cat("\n------------------------------------------------------------------------------------\n")
cat(" HT.2: Yonelinas (1994), Exp.2, long lists")
cat("\n------------------------------------------------------------------------------------\n")
Opti1.Obj <- SDT.Estimate(data = data.long, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = T)
Stat1.Obj <- SDT.Statistics(Opti1.Obj)
print(Stat1.Obj)

#===============================================================================
#
# 4. HT.2 MODEL: FREE VARIANCE
#
#===============================================================================
fixed.exc.inc <- matrix(c(rep(0, 8),                       # Fixed constraints exclusion / inclusion
                              13:16, 20:23), nr = 2, byrow = T)
                        
fixed <- cbind(fixed.dist, fixed.exc.inc)
ident <- matrix(c(11:12,    8:9, 10,
                  25:24,  17:18, 19), nr = 2, byrow = T)

cat("\n------------------------------------------------------------------------------------\n")
cat(" HT.2: Yonelinas (1994), Exp.2, short lists, FREE VARIANCE PARAMETER")
cat("\n------------------------------------------------------------------------------------\n")
Opti1.Obj <- SDT.Estimate(data = data.short, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = T)
Stat1.Obj <- SDT.Statistics(Opti1.Obj)
print(Stat1.Obj)

cat("\n------------------------------------------------------------------------------------\n")
cat(" HT.2: Yonelinas (1994), Exp.2, long lists, FREE VARIANCE PARAMETER")
cat("\n------------------------------------------------------------------------------------\n")
Opti1.Obj <- SDT.Estimate(data = data.long, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = T)
Stat1.Obj <- SDT.Statistics(Opti1.Obj)
print(Stat1.Obj)
