# ================================================================
# Examples of the documentation of Model HT.n:
# HT.n: SDT model with guessing
# Data from Hilford et al. (2002)
#
# Date of creation: February, 2013
# Author: Siegfried Macho
# ================================================================
library(numDeriv)

# DIRECTORIES (PLEASE ADJUST TO YOUR PERSONAL SETTING)
DirSource <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/"			
DirSourceM <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/Models/"

# FILE NAME WITH THE MODEL 
Model     <- "SDT-HT-n.R"
Model.SDT <- "SDT-SDT.R"
# FILE NAMES FOR ESTIMATION 
Main      <- "SDT-Main.R"
Auxiliary <- "SDT-Auxiliary.R"

# LOAD SOURCE FILES WITH SDT MODEL
source(paste(DirSourceM, Model, sep = ""))
# LOAD SOURCE FILES WITH SDT MODEL
source(paste(DirSourceM, Model.SDT, sep = ""))

# LOAD SOURCE FILES WITH AUXILIARY AND ESTIMATION FUNCTIONS
source(paste(DirSource, Auxiliary, sep = ""))
source(paste(DirSource, Main, sep = ""))

# DATA: HILFORD ET AL. (2002), Exp.1
data.1 <- c(264, 133, 159, 116,  80,  33,			# Male voice 
 	       25,  62, 105, 128, 145, 294)			# Female voice


# DATA: HILFORD ET AL. (2002), Exp.2
data.2 <- c(1358, 550, 701, 585, 466,  300,		# Male
 	       278, 477, 598, 614, 589, 1404)		# Female


# DATA: HILFORD ET AL. (2002), Exp.3
data.3 <- c(1313, 538, 524, 529, 363,  333,		# Top 
 	       348, 356, 455, 552, 510, 1379)		# Bottom

n <- 2
TF <- print.detail <- F                               # Flag indicating output

#===============================================================================
# SDT - Model with Guessing
#===============================================================================
# Specification of constraints
fixed.A <- matrix(c(0, 0, 0.5, 0.5,  0,  0,                         # Source A
                    2, 3,   4,   5,  6,  7), nr = 2, byrow = T)
                         
fixed.B <- matrix(c( 1,  0,  0, 0.5, 0.5,  0,  0,                   # Source B
                     9, 11, 12,  13,  14, 15, 16), nr = 2, byrow = T)
                        
fixed <- cbind(fixed.A, fixed.B)
ident <- matrix(c( 1,
                  10), nr = 2, byrow = T)

# Compute start parameter vector
par <- SDT.HT.n.start.par(n, data.1)

# Add redundant parameters so that their positions can be seen in the output
fixed.show <- HT.n.Redundant.to.Fixed(par, n, fixed, ident)

# Compute Parameter information
cat("\n------------------------------------------------------------------------------------\n")
cat(" HT.n: Configuration of parameters for fitting SDT model with guessing")
cat("\n------------------------------------------------------------------------------------\n")
Par.Info <- SDT.Parameter.Info(par = par, n = n, Model.Id = "HT.n", fixed = fixed.show, ident = ident)
print(Par.Info)




Opti1.Obj <- SDT.Estimate(data = data.1, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = TF)
Stat1.Obj <- SDT.Statistics(Opti1.Obj)
if(print.detail)
{
      cat("\n------------------------------------------------------------------------------------\n")
      cat(" HT.n: Hilford et al. (2002): Exp.1")
      cat("\n------------------------------------------------------------------------------------\n")
      print(Stat1.Obj)
}

Opti2.Obj <- SDT.Estimate(data = data.2, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = TF)
Stat2.Obj <- SDT.Statistics(Opti2.Obj)
if(print.detail)
{
      cat("\n------------------------------------------------------------------------------------\n")
      cat(" HT.n: Hilford et al. (2002): Exp.2")
      cat("\n------------------------------------------------------------------------------------\n")
      print(Stat2.Obj)
}


Opti3.Obj <- SDT.Estimate(data = data.3, n = n, Model.Id = "HT.n", fixed = fixed, ident = ident, test = TF)
Stat3.Obj <- SDT.Statistics(Opti3.Obj)
if(print.detail)
{
      cat("\n------------------------------------------------------------------------------------\n")
      cat(" HT.n: Hilford et al. (2002): Exp.3")
      cat("\n------------------------------------------------------------------------------------\n")
      print(Stat3.Obj)
}

#================================
# SDT - Model without guessing
#================================
SDT.Opti1.Obj <- SDT.Estimate(data = data.1, test = TF)
SDT.Stat1.Obj <- SDT.Statistics(SDT.Opti1.Obj)
if(print.detail)
{
      cat("\n------------------------------------------------------------------------------------\n")
      cat(" SDT: Hilford et al. (2002): Exp.1")
      cat("\n------------------------------------------------------------------------------------\n")
      print(SDT.Stat1.Obj)
}

SDT.Opti2.Obj <- SDT.Estimate(data = data.2, test = TF)
SDT.Stat2.Obj <- SDT.Statistics(SDT.Opti2.Obj)
if(print.detail)
{
      cat("\n------------------------------------------------------------------------------------\n")
      cat(" SDT: Hilford et al. (2002): Exp.2")
      cat("\n------------------------------------------------------------------------------------\n")
      print(SDT.Stat2.Obj)
}

SDT.Opti3.Obj <- SDT.Estimate(data = data.3, test = TF)
SDT.Stat3.Obj <- SDT.Statistics(SDT.Opti3.Obj)
if(print.detail)
{
      cat("\n------------------------------------------------------------------------------------\n")
      cat(" SDT: Hilford et al. (2002): Exp.3")
      cat("\n------------------------------------------------------------------------------------\n")
      print(SDT.Stat3.Obj)
}


#================================
# Extract Fit information
#================================
pos <- c(3, 4, 6)
Erg <- c(Stat1.Obj$Statistics[pos], SDT.Stat1.Obj$Statistics[pos])
Erg <- rbind(Erg, c(Stat2.Obj$Statistics[pos], SDT.Stat2.Obj$Statistics[pos]))
Erg <- rbind(Erg, c(Stat3.Obj$Statistics[pos], SDT.Stat3.Obj$Statistics[pos]))
colnames(Erg) <- c("Guessing-G2",  "Guessing-df", "Guessing-p", "SDT-G2", "SDT-df", "SDT-p")
rownames(Erg) <- paste("Exp.", 1:3, sep = "")

cat("\n------------------------------------------------------------------------------------\n")
cat(" Comparison of SDT with and without guessing: Hilford et al. (2002): Exp.1-3")
cat("\n------------------------------------------------------------------------------------\n")
print(Erg)