# ================================================================
# MIX-2.1: Examples of the documentation of Model MIX.2:
# Modeling source monitoring data from Hilford et al. (2002)
#
# Date of creation: May, 2009
# Author: Siegfried Macho
# ================================================================
library(numDeriv)

# DIRECTORIES (PLEASE ADJUST TO YOUR PERSONAL SETTING)
DirSource <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/"			
DirSourceM <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/Models/"

# FILE NAME WITH THE MODEL 
Model <- "SDT-MIX-2.R"

# FILE NAMES FOR ESTIMATION 
Main      <- "SDT-Main.R"
Auxiliary <- "SDT-Auxiliary.R"

# LOAD SOURCE FILES WITH SDT MODEL
source(paste(DirSourceM, Model, sep = ""))

# LOAD SOURCE FILES WITH AUXILIARY AND ESTIMATION FUNCTIONS
source(paste(DirSource, Auxiliary, sep = ""))
source(paste(DirSource, Main, sep = ""))

# DATA: HILFORD ET AL. (2002), Exp.1
data.1 <- c(264, 133, 159, 116,  80,  33,			# Male voice 
 	       25,  62, 105, 128, 145, 294)			# Female voice


# DATA: HILFORD ET AL. (2002), Exp.2
data.2 <- c(1358, 550, 701, 585, 466,  300,		# Male
 	       278, 477, 598, 614, 589, 1404)		# Female


# DATA: HILFORD ET AL. (2002), Exp.3
data.3 <- c(1313, 538, 524, 529, 363,  333,		# Top 
 	       348, 356, 455, 552, 510, 1379)		# Bottom

# 1. FIXED CONSTRAINTS		
# 1.1 FIX MEANS OF SECOND DISTRIBUTION OF EACH MIXTURE TO 0 
fixed.m <- matrix(c(0,    0,
	   	        4, 4+10), nr = 2, byrow = T)
# 1.2 FIX ALL SD TO 1.0
fixed.sd <- matrix(c(1,  1,    1,    1,
	   	         3,  5, 3+10, 5+10), nr = 2, byrow = T)

# 1.3 FUNCTIONAL CONSTRAINTS ON MEANS (MUST BE INCLUDED)
fixed.fct <- matrix(c(2,
	   	          2), nr = 2, byrow = T)

# 1.4 COMBINE FIXED CONSTRAINTS
fixed <- cbind(fixed.m, fixed.sd, fixed.fct)

# 2. EQUALITY CONSTRAINTS: IDENTICAL PROBABILITIES AS WELL AS IDENTICAL THRESHOLDS
ident <- matrix(c( 1, 6:10,
	   	      11, 6:10+10), nr = 2, byrow = T)

# 3. FUNCTIONAL CONSTRAINT: m.A = -m.B (first distribution)
fct.m <- function(par)
{
	tindex <- 2					# Target index: m.A
	sindex <- 2 + 10				# Source index: m.B

	par[tindex] <- -par[sindex]

	# Define Gradient function
	gr.fct <- function(grad, par)
	{
		grad[sindex] <- grad[sindex] - grad[tindex]
		return(grad)
	}
	attr(par, "gradient") <- gr.fct			# Gradient
	
	return(par)
}
# 4 DEFINE FUNCTION TO COMPUTE Jacobian AND ASSIGN IT TO THE FUNCTION
J.fct <- function(par)
{
      J <- diag(length(par))
      J[2, 12] <- -1
      J
}   
attr(fct.m, "Jacobian") <- J.fct			# Jacobian matrix

cat("\n------------------------------------------------------------------------------------\n")
cat(" MIX-2: Hilford et al. (2002): Exp.1")
cat("\n------------------------------------------------------------------------------------\n")
Opti1.Obj <- SDT.Estimate(data = data.1, Model.Id = "MIX.2", fixed = fixed, ident = ident, functional = fct.m, test = T)
Stat1.Obj <- SDT.Statistics(Opti1.Obj)
print(Stat1.Obj)

cat("\n------------------------------------------------------------------------------------\n")
cat(" MIX-2: Hilford et al. (2002): Exp.2")
cat("\n------------------------------------------------------------------------------------\n")
Opti2.Obj <- SDT.Estimate(data = data.2, Model.Id = "MIX.2", fixed = fixed, ident = ident, functional = fct.m, test = T)
Stat2.Obj <- SDT.Statistics(Opti2.Obj)
print(Stat2.Obj)

cat("\n------------------------------------------------------------------------------------\n")
cat(" MIX-2: Hilford et al. (2002): Exp.3")
cat("\n------------------------------------------------------------------------------------\n")
Opti3.Obj <- SDT.Estimate(data = data.3, Model.Id = "MIX.2", fixed = fixed, ident = ident, functional = fct.m, test = T)
Stat3.Obj <- SDT.Statistics(Opti3.Obj)
print(Stat3.Obj)

# PLOT GAUSSIAN DISTRIBUTIONS AND THRESHOLDS, AS WELL AS ROC CURVES FOR EXP.3:
#------------------------------------------------------------------------
# COMMENT ON THE ORDERING OF GAUSSIAN MODELS:
# The ordering is: 1. First  model of Signal 1 (m = -1.745, sd = 1.0)
#                  2. Second model of Signal 1 (m =      0, sd = 1.0)
#                  3. First  model of Signal 2 (m =  1.745, sd = 1.0)
#                  4. Second model of Signal 2 (m =      0, sd = 1.0)
#
# The second model of Signal 1 is masked be the second model of Signal 2
#------------------------------------------------------------------------
# PLOT DENSITY CURVES
SDT.Plot(Opti3.Obj, cols = c(4, 3, 6, 3), SDT.legend = list(text = c("left", "no info", "right"), cols = c(4, 3, 6)))

# PLOT ROC CURVES
# Plot ROC curve with data
SDT.Plot(Opti3.Obj, cols = 3, labels = c("False Alarms", "Hits"), option = "ROC+")

# Plot z-ROC curve with data
SDT.Plot(Opti3.Obj, cols = 3, labels = c("z(FA)", "z(H)"), option = "zROC+")