# ================================================================
# Examples of the documentation of Model MIX-PD:
# MIX-PD.2-2: Modeling Associative Recognition
# Data from Kelley and Wixted, Exp.2
#
# Date of creation: August, 2009
# Author: Siegfried Macho
# ================================================================
library(numDeriv)

# DIRECTORIES (PLEASE ADJUST TO YOUR PERSONAL SETTING)
DirSource <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/"			
DirSourceM <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/Models/"

# FILE NAME WITH THE MODEL 
Model <- "SDT-MIX-PD.R"

# FILE NAMES FOR ESTIMATION 
Main      <- "SDT-Main.R"
Auxiliary <- "SDT-Auxiliary.R"

# LOAD SOURCE FILES WITH SDT MODEL
source(paste(DirSourceM, Model, sep = ""))

# LOAD SOURCE FILES WITH AUXILIARY AND ESTIMATION FUNCTIONS
source(paste(DirSource, Auxiliary, sep = ""))
source(paste(DirSource, Main, sep = ""))


# DATA OF KELLEY & WIXTED (2001), Experiment 2
data <- c(879, 553, 345,  61,  54,  28,	# Exp.2: Lure items
	    179, 109,  82,  45,  28,  37,   # Exp.2: Weak   Items, rearranged
	     81,  69,  62,  31,  37, 200,	# Exp.2: Weak   Items, intact
	    215,  82,  60,  31,  31,  61,	# Exp.2: Strong Items, rearranged
 	     21,  24,  33,  21,  33, 348)	# Exp.2: Strong Items, intact

# NUMBER OF CONDITIONS (TYPES OF SIGNALS)
n <- 5

# IDENTITY CONSTRAINTS. p.left = p.right, d'.left = d'.right
ident <- matrix(c(3, 5, 3+6, 5+6,
	   	      4, 6, 4+6, 6+6), nr = 2, byrow = T)

# IDENTITY CONSTRAINTS. d'.left = d'.right
ident.d <- matrix(c(5, 5+6,
	   	        6, 6+6), nr = 2, byrow = T)

# IDENTITY CONSTRAINTS. d'.left = d'.right (strong = weak)
ident.d.w.s <- matrix(c(5,   5,   5,
	     	            6, 5+6, 6+6), nr = 2, byrow = T)

# IDENTITY CONSTRAINTS. m (weak) = m (strong) sd (weak) = sd (strong)
ident.m.sd <- matrix(c(1,   2,
	     	           7, 2+6), nr = 2, byrow = T)


cat("\n--------------------------------------------------------------------\n")
cat(" MIX.PD: Associative recognition (Data from Kelley and Wixted, Exp.2:")
cat("\n p.left  = p.right, d'.left  = d'.right") 
cat("\n--------------------------------------------------------------------\n")
Opti.Obj <- SDT.Estimate(data = data, Model.Id = "MIX.PD", n = n, ident = ident, test = T)
Stat.Obj <- SDT.Statistics(Opti.Obj)
print(Stat.Obj)

cat("\n--------------------------------------------------------------------\n")
cat(" MIX.PD: Associative recognition (Data from Kelley and Wixted, Exp.2):")
cat("\n p.left  <> p.right, d'.left  = d'.right") 
cat("\n--------------------------------------------------------------------\n")
Opti.Obj <- SDT.Estimate(data = data, Model.Id = "MIX.PD", n = n, ident = ident.d, test = T)
Stat.Obj <- SDT.Statistics(Opti.Obj)
print(Stat.Obj)

cat("\n--------------------------------------------------------------------\n")
cat(" MIX.PD: Associative recognition (Data from Kelley and Wixted, Exp.2):")
cat("\n p.left  <> p.right, d'.left  = d'.right (weak = strong)") 
cat("\n--------------------------------------------------------------------\n")
Opti.Obj <- SDT.Estimate(data = data, Model.Id = "MIX.PD", n = n, ident = ident.d.w.s, test = T)
Stat.Obj <- SDT.Statistics(Opti.Obj)
print(Stat.Obj)

# PLOT GAUSSIAN DISTRIBUTIONS AND THRESHOLDS, AS WELL AS ROC CURVES:
# PLOT DENSITY CURVES
SDT.Plot(Opti.Obj, cols = c(6, 3, 3, 3, 4, 4, 4), SDT.legend = list(text = c("New", paste("Weak", 1:3),paste("Strong", 1:3)), cols = c(6, 3, 3, 3, 4, 4, 4)))

# PLOT ROC CURVES
# Plot ROC curve with data
SDT.Plot(Opti.Obj, cols = c(3, 3, 4, 4), ltys = c(2, 1, 2, 1), labels = c("False Alarms", "Hits"), option = "ROC+",
SDT.legend = list(text = c("weak rearranged", "weak intact","strong rearranged", "strong intact"), cols = c(3, 3, 4, 4), ltys = c(2, 1, 2, 1)))

# Plot z-ROC curve with data
SDT.Plot(Opti.Obj, cols = c(3, 3, 4, 4), ltys= c(2, 1, 2, 1), labels = c("z(FA)", "z(H)"), option = "zROC+",
SDT.legend = list(text = c("weak rearranged", "weak intact","strong rearranged", "strong intact"), cols = c(3, 3, 4, 4), ltys = c(2, 1, 2, 1)))

cat("\n--------------------------------------------------------------------\n")
cat(" MIX.PD: Associative recognition (Data from Kelley and Wixted, Exp.2):")
cat("\n p.left  <> p.right, d'.left  = d'.right (weak = strong)") 
cat("\n m (weak) = m (strong), sd (weak) = sd (strong)") 
cat("\n--------------------------------------------------------------------\n")
Opti.Obj <- SDT.Estimate(data = data, Model.Id = "MIX.PD", n = n, ident = cbind(ident.d.w.s, ident.m.sd), test = T)
Stat.Obj <- SDT.Statistics(Opti.Obj)
print(Stat.Obj)

cat("\n---------------------------------------------------------------------------------------------\n")
cat(" MIX-PD: Associative recognition (Data from Kelley and Wixted, Exp.2): Parameter Information")
cat("\n--------------------------------------------------------------------------------------------\n")
# COMPUTE AND PRINT PARAMETER INFORMATION
pinfo <- SDT.Parameter.Info(data = Opti.Obj$SDT.data, par = Opti.Obj$SDT.full.par, n = Opti.Obj$SDT.n, Model.Id = Opti.Obj$SDT.Model.Id, 
                              fixed = Opti.Obj$SDT.fixed, ident = Opti.Obj$SDT.ident)
print(pinfo)