# ======================================================================
# SDT-2D-10-1: Implementation of the CDP model of Wixted an Mickes (2010)
#
# Data: Rotello et al. (2005, p.873)
#
# Author: Siegfried Macho
# Date of last update: September, 2013
# =============================================================
library(ellipse)

# DIRECTORIES (my computer, PLEASE ADJUST TO YOUR PERSONAL SETTING)
DirSource <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/"			
DirSourceM <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/Models/"

# FILE NAME WITH THE MODEL 
Model <- "SDT-SDT-2D.R"

# FILE NAMES FOR ESTIMATION 
Main      <- "SDT-Main.R"
Auxiliary <- "SDT-Auxiliary.R"

# LOAD SOURCE FILES WITH AUXILIARY AND ESTIMATION FUNCTIONS
source(paste(DirSource, Auxiliary, sep = ""))
source(paste(DirSource, Main, sep = ""))

# LOAD SOURCE FILES WITH SDT-2D MODEL
source(paste(DirSourceM, Model, sep = ""))

# DATA from Rotello et al.(2005), NEUTRAL CONDITION
data <- matrix(c(
          # Lure	Target
            745,	290,	# Category 1 New
            0,	0,	# Category 1 New
            199,	122,	# Category 2 Know
            46,	26,	# Category 2 Remember
            124,	106,	# Category 3 Know
            29,	38,	# Category 3 Remember
            84,	101,	# Category 4 Know
            47,	61,	# Category 4 Remember
            45,	92,	# Category 5 Know
            51,	109,	# Category 5 Remember
            13,	59,	# Category 6 Know
            55,	433),	# Category 6 Remember
            nc = 2, byrow = T)

dvec <- c(data[,1], data[,2])           # Make data vector

# SPECIFICATION OF A MATRIX FOR POOLING DATA
R <- diag(length(data))
R[1, 2]   <- 1
# Matrix: Source cells in columns, target cells in rows
R[13, 14] <- 1
# Delete rows of cells that are grouped
R <- R[-c(2, 14),]

# CONFIGURATION OF THE MODEL
cfg <- list(n.sdt = 2, k1 = 5, k2 = 1, ds = 0, standard = F, R = R)

# SPECIFICATION OF FIXED CONSTRAINTS:
fixed.par <-matrix(c(0, 0, sqrt(2), 1, 1/sqrt(2), 1,  1,
                     1, 2,       3, 4,         5, 8, 10), nrow = 2, byrow = T)  

#---------------------------------------------------------------------------
# DEFINE FUNCTION FOR IMPLEMENTING FUNCTIONAL CONSTRAINTS
#---------------------------------------------------------------------------
fct.res <- function(par)
{
      par[8]  <- sqrt(1 + par[9]*par[9])
      par[10] <- par[9]/par[8]
      
      # Gradient function
      gr.fct <- function(grad, par)
      {
            zw <- sqrt(1+ par[9]*par[9])
            grad[9] <- grad[9] + grad[8]*par[9]/zw + grad[10]*(1/zw - par[9]*par[9]/zw/zw/zw)
            return(grad)
      }
      attr(par, "SDT.gradient") <- gr.fct           # Gradient function
      return(par)
}

#---------------------------------------------------------------------------
# DEFINE FUNCTION FOR COMPUTING THE JACOBIAN FOR THE ANALYTICAL COMPUTATION
# OF STANDARDIZED RESIDUALS
#---------------------------------------------------------------------------
J.fct <- function(par)
{
      zw <-  sqrt(1+ par[9]*par[9])
      J <- diag(length(par))
      J[8, 9]  <- par[9]/zw
      J[10, 9] <- 1/zw - par[9]*par[9]/zw/zw/zw
      J
}
attr(fct.res, "SDT.Jacobian") <- J.fct

# CALL ESTIMATION MODULE AND PRINT THE RESULT
Opti.obj <- SDT.Estimate(data = dvec, n = cfg, Model.Id = "SDT.2D", fixed = fixed.par, functional = fct.res, test = T)

# COMPUTE AND PRINT STATISTICS
Stat.Obj <- SDT.Statistics(Opti.obj)
cat("\n==================================================\n")
cat(" Results for the data of the neutral condition:")
cat("\n==================================================\n")
print(Stat.Obj)

# PLOT CONFIRGUATION
xy.labels <- c("Confidence", "Recollection")
SDT.Plot(Opti.obj, cols = 3:6, labels = xy.labels, option = 11)

#===============================================================================
# MODELING OF THE CONSCERVATIVE CONDITION
#===============================================================================

# DATA from Rotello et al.(2005), CONSERVATIVE CONDITION
data <- matrix(c(
          # Lure	Target
            870,	394,	# Category 1 New
            0,	0,	# Category 1 New
            200,	160,	# Category 2 Know
            11,	5,	# Category 2 Remember
            150,	126,	# Category 3 Know
            8,	8,	# Category 3 Remember
            65,	104,	# Category 4 Know
            9,	19,	# Category 4 Remember
            42,	82,	# Category 5 Know
            26,	49,	# Category 5 Remember
            30,	100,	# Category 6 Know
            25,	388),	# Category 6 Remember
            nc = 2, byrow = T)

dvec <- c(data[,1], data[,2])           # Make data vector

# CALL ESTIMATION MODULE AND PRINT THE RESULT
Opti.obj <- SDT.Estimate(data = dvec, n = cfg, Model.Id = "SDT.2D", fixed = fixed.par, functional = fct.res, test = T)

# COMPUTE AND PRINT STATISTICS
Stat.Obj <- SDT.Statistics(Opti.obj)
cat("\n=====================================================\n")
cat(" Results for the data of the conservative condition:")
cat("\n=====================================================\n")
print(Stat.Obj)

# PLOT CONFIGURATION
xy.labels <- c("Confidence", "Recollection")
SDT.Plot(Opti.obj, cols = 3:6, labels = xy.labels, option = 11)
