# ======================================================================
# SDT-2D-11: Implementation of the Criterion Variability Model
#            of Wixted an Mickes (2010)
#
# Data: Rotello et al. (2005, p.873)
#
# Author: Siegfried Macho
# Date of last update: September, 2013
# =============================================================
library(ellipse)

# DIRECTORIES (my computer, PLEASE ADJUST TO YOUR PERSONAL SETTING)
DirSource <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/"			
DirSourceM <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/Models/"

# FILE NAME WITH THE MODEL 
Model <- "SDT-SDT-2D.R"

# FILE NAMES FOR ESTIMATION 
Main      <- "SDT-Main.R"
Auxiliary <- "SDT-Auxiliary.R"

# NAME OF THE DATA FILE
Data <- "Daten (Rotello, 2005, Neutral).txt"

# LOAD SOURCE FILES WITH SDT-2D MODEL
source(paste(DirSourceM, Model, sep = ""))

# LOAD SOURCE FILES WITH AUXILIARY AND ESTIMATION FUNCTIONS
source(paste(DirSource, Auxiliary, sep = ""))
source(paste(DirSource, Main, sep = ""))

# DATA from Rotello et al.(2005), NEUTRAL CONDITION
data <- matrix(c(
          # Lure	Target
            0,	0,	# Category 1 New
            745,	290,	# Category 1 New
            46,	26,	# Category 2 Remember
            199,	122,	# Category 2 Know
            29,	38,	# Category 3 Remember
            124,	106,	# Category 3 Know
            47,	61,	# Category 4 Remember
            84,	101,	# Category 4 Know
            51,	109,	# Category 5 Remember
            45,	92,	# Category 5 Know
            55,	433,	# Category 6 Remember
            13,	59),	# Category 6 Know
            nc = 2, byrow = T)

dvec <- c(data[,1], data[,2])           # Make data vector

# SPECIFICATION OF A MATRIX FOR POOLING DATA
R <- diag(length(dvec))
R[1, 2]   <- 1
# Matrix: Source cells in columns, target cells in rows
R[13, 14] <- 1

# Delete rows of cells that are grouped
R <- R[-c(2, 14),]

# CONFIGURATION OF THE MODEL
cfg <- list(n.sdt = 2, k1 = 5, k2 = 1, ds = 0, standard = F, R = R)

# SPECIFICATION OF FIXED CONSTRAINTS: Standard deviation to 1
fixed.par <-matrix(c(0, 1, 0, 0, 1,  0,  0,
                     1, 3, 5, 7, 9, 10, 16), nrow = 2, byrow = T)  

#---------------------------------------------------------------------------
# DEFINE FUNCTION FOR IMPLEMENTING FUNCTIONAL CONSTRAINTS
#---------------------------------------------------------------------------
fct.res <- function(par)
{
      zw      <-  par[4]*par[4] - 1 + par[8]*par[8]
      if(zw <= 0) par[9] <- 10e-12
      else        par[9] <- sqrt(zw)
      par[7]  <-  par[2] - par[6]
      par[5]  <- -1/par[4]
      par[10] <- -par[8]/par[9]
      
      # Gradient function
      gr.fct <- function(grad, par)
      {
            grad[2] <- grad[2] + grad[7]
            grad[6] <- grad[6] - grad[7]
            grad[4] <- grad[4] + grad[5]/par[4]/par[4] + grad[9]*par[4]/par[9] + grad[10]*par[4]*par[8]/par[9]/par[9]/par[9]
            grad[8] <- grad[8] + grad[9]*par[8]/par[9] + grad[10]*(-1/par[9] + par[8]*par[8]/par[9]/par[9]/par[9])
            return(grad)
      }
      attr(par, "SDT.gradient") <- gr.fct           # Gradient function
      return(par)
}

#---------------------------------------------------------------------------
# DEFINE FUNCTION FOR COMPUTING THE JACOBIAN FOR THE ANALYTICAL COMPUTATION
# OF STANDARDIZED RESIDUALS
#---------------------------------------------------------------------------
J.fct <- function(par)
{
      zw <-  sqrt(1+ par[9]*par[9])
      J <- diag(length(par))
      J[7, 2] <- 1
      J[7, 6] <- -1
      J[5, 4] <- 1/par[4]/par[4]
      J[9, 4] <- par[4]/par[9]
      J[10,4] <- par[4]*par[8]/par[9]/par[9]/par[9]
      J[9, 8] <- par[8]/par[9]
      J[10,8] <- -1/par[9] + par[8]*par[8]/par[9]/par[9]/par[9]
      J
}
attr(fct.res, "SDT.Jacobian") <- J.fct

# START PARAMETER (BASED ON VALUES REPORTED BY WIXTED AND MICKES(2010))
par.neutral <- c(   0,       1.28,    1,  sqrt(1+0.96^2)     , -1/sqrt(1+0.96^2), 
                 1.15, -1.15+1.28, 1.33,  sqrt(1.33^2+0.96^2), -1.33/sqrt(1.33^2+0.96^2),
                 0.05, 0.48, 0.82, 1.20, 1.68, 0)

# CALL ESTIMATION MODULE AND PRINT THE RESULT
Opti.obj <- SDT.Estimate(par = par.neutral, data = dvec, n = cfg, Model.Id = "SDT.2D", fixed = fixed.par, functional = fct.res, test = T)

# COMPUTE AND PRINT STATISTICS
Stat.Obj <- SDT.Statistics(Opti.obj)
cat("\n==================================================\n")
cat(" Results for the data of the neutral condition:")
cat("\n==================================================\n")
print(Stat.Obj)

# PLOT CONFIRGUATION
xy.labels <- c("Confidence", "Recollection - Confidence")
SDT.Plot(Opti.obj, cols = 3:6, labels = xy.labels, option = 11)

#===============================================================================
# MODELING OF THE CONSCERVATIVE CONDITION
#===============================================================================

# DATA from Rotello et al.(2005), CONSERVATIVE CONDITION
data <- matrix(c(
          # Lure	Target
            0,	0,	# Category 1 New
            870,	394,	# Category 1 New
            11,	5,	# Category 2 Remember
            200,	160,	# Category 2 Know
            8,	8,	# Category 3 Remember
            150,	126,	# Category 3 Know
            9,	19,	# Category 4 Remember
            65,	104,	# Category 4 Know
            26,	49,	# Category 5 Remember
            42,	82,	# Category 5 Know
            25,	388,	# Category 6 Remember
            30,	100),	# Category 6 Know
            nc = 2, byrow = T)

dvec <- c(data[,1], data[,2])           # Make data vector

# START PARAMETER (BASED ON VALUES REPORTED BY WIXTED AND MICKES(2010))
par.conservative <- c(   0,       1.92,    1,  sqrt(1+0.74^2)     , -1/sqrt(1+0.74^2), 
                 1.16, -1.16+1.92, 1.51,  sqrt(1.51^2+0.74^2), -1.51/sqrt(1.51^2+0.74^2),
                 0.27, 0.70, 1.08, 1.39, 1.77, 0)

# CALL ESTIMATION MODULE AND PRINT THE RESULT
Opti.obj <- SDT.Estimate(par = par.conservative, data = dvec, n = cfg, Model.Id = "SDT.2D", fixed = fixed.par, functional = fct.res, test = T)

# COMPUTE AND PRINT STATISTICS
Stat.Obj <- SDT.Statistics(Opti.obj)
cat("\n=====================================================\n")
cat(" Results for the data of the conservative condition:")
cat("\n=====================================================\n")
print(Stat.Obj)

# PLOT CONFIGURATION
xy.labels <- c("Confidence", "Recollection - Confidence")
SDT.Plot(Opti.obj, cols = 3:6, labels = xy.labels, option = 11)
