# =============================================================
# SDT-2D-5-2: Example of the SDT-2D MODEL
#             Estimation of data from Olzak & Kramer (1984)
#             - No Violations of decisional separability
#             - Functional cosntraints implementing a diamond confirguration
#             - Robust estimation of thresholds
#
# Author: Siegfried Macho
# Date of last update: September, 2013
# =============================================================
library(ellipse)

# DIRECTORIES (my computer, PLEASE ADJUST TO YOUR PERSONAL SETTING)
DirSource <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/"			
DirSourceM <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/R-Code/Models/"
DirData <- "C:/Documents and Settings/macho/Statistik/_SDT-Models (Version 2)/Data/"

# FILE NAME WITH THE MODEL 
Model <- "SDT-SDT-2D.R"

# FILE NAMES FOR ESTIMATION 
Main      <- "SDT-Main.R"
Auxiliary <- "SDT-Auxiliary.R"

# NAME OF THE DATA FILE
Data <- "Daten (Wickens, 1992).txt"

# LOAD SOURCE FILES WITH SDT-2D MODEL
source(paste(DirSourceM, Model, sep = ""))

# LOAD SOURCE FILES WITH AUXILIARY AND ESTIMATION FUNCTIONS
source(paste(DirSource, Auxiliary, sep = ""))
source(paste(DirSource, Main, sep = ""))

# LOAD DATA
data <- read.table(paste(DirData, Data, sep = ""), header = T)
data <- c(data[,3], data[,4], data[,5], data[,6])

# CONFIGURATION OF THE MODEL
cfg <- list(n.sdt = 4, k1 = 5, k2 = 5, robust = T, struct.zero =  c(1, 36+1, 2*36+1, 3*36+1))

# SPECIFICATION OF FUNCTIONAL CONSTRAINTS
pos <- c(12, 8, 14, 17, 19, 16, 18)
values <- rep(-1, length(pos))
fixed.par <- matrix (c(values, pos), nrow = 2, byrow = T)

# Define function for specifying equality constraints
funct <- function(par)
{
	par[c(12, 8, 14, 17, 19, 18)] <- par[c(2, 3, 4, 7, 9, 13)]
	par[16] <- par[6] + par[11]							# x-coordinate of D(1, 1) = x of D(0, 1)  +  x of D(1, 0)

	# Define Gradient function
	gr.fct <- function(grad, par)
	{
		grad[c(2:4, 7, 9, 13)] <- grad[c(2:4, 7, 9, 13)] + grad[c(12, 8, 14, 17, 19, 18)]

		grad[6]  <- grad[6]  + grad[16]
		grad[11] <- grad[11] + grad[16]

		return(grad)
	}
	attr(par, "SDT.gradient") <- gr.fct			# Gradient
	return(par)
}

# Specify function for computing the Jacobian representing functional constraints
# Note: Target parameters make up the rows and source parameters make up the columns of the Jacobian
J.fct <- function(full.par)
{
      J <- diag(length(full.par))
      J[12,  2]  <- 1
      J[ 8,  3]  <- 1
      J[14,  4]  <- 1
      J[17,  7]  <- 1
      J[19,  9]  <- 1
      J[18, 13]  <- 1
        
      J[16,  6]  <- 1
      J[16, 11]  <- 1
      J
}
 attr(funct, "SDT.Jacobian") <- J.fct


# PRINT PARAMETER INFORMATION
p.info <- SDT.Parameter.Info(data = data, n = cfg, Model.Id = "SDT.2D")
cat("\n==============================================\n")
cat(" Parameter information:")
cat("\n==============================================\n")                     
print(p.info)

# CALL ESTIMATION MODULE AND PRINT THE RESULT
Opti.obj <- SDT.Estimate(data = data, n = cfg, Model.Id = "SDT.2D", fixed = fixed.par, functional = funct, test = T)

# COMPUTE AND PRINT STATISTICS
Stat.Obj <- SDT.Statistics(Opti.obj)
cat("\n=======================================================================================================\n")
cat(" Results for the data of Olzak and Kramer (1984): Functional constraints (diamond configuration):")
cat("\n=======================================================================================================\n")
print(Stat.Obj)

# PLOT CONFIGURATION
xy.labels <- c("Low Frequency Signal", "High Frequency Signal")
SDT.Plot(Opti.obj, cols = 3:6, labels = xy.labels)